/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.kafka.connect.runtime.rest.entities;

import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class ConnectorTypeTest {

    @Test
    public void testToStringIsLowerCase() {
        for (ConnectorType ct : ConnectorType.values()) {
            String shouldBeLower = ct.toString();
            assertFalse(shouldBeLower.isEmpty());
            for (Character c : shouldBeLower.toCharArray()) {
                assertTrue(Character.isLowerCase(c));
            }
        }
    }

    @Test
    public void testForValue() {
        for (ConnectorType ct : ConnectorType.values()) {
            assertEquals(ct, ConnectorType.forValue(ct.toString()));
        }
    }
}
